/*
This is an unsupported product. It was developed as part of a study at Ostfold
Regional College, Norway in the spring of 1992.

You use this software at your own risk. Neither the authors nor the Ostfold
Regional College nor any other person or organization except yourself is
responsible for any damage or loss of data derived from the use of this
software. You may distribute this software freely as long as it is in its
original form and all of its files are included. If you make any changes to any
part of this software or the files distributed with it, please state so and do
NOT distribute it under its original name.
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include "nntp_tcp.h"
#include "div_func.h"


/*
-----------------------------------------------------------------------------
Function name: get_article()
-----------------------------------------------------------------------------
Parameters:	int f		A file descriptor
		char *bf	A pointer to a buffer.
Return value:	0 on error
		1 on success
Called by:	do_innlegg()
Calls:		none
Description:	This function gets the article id from a file. The id is
		contained between a <>. It puts the id in bf.
-----------------------------------------------------------------------------
*/

int get_article(f, bf)
int f;
char *bf;
{
	char ch[2];
	int ret = 1;
	int ret2 = 1;
	int teller = 0;

	while(ret2 >0)
	{	
		ret2 = read(f, ch, 1);
		if( ch[0] == '<' )
		{
			while(ret >0 )
			{
				bf[teller++] = ch[0];	
				if(ch[0] == '>') {
					bf[teller] = '\0';
					return(1);
				}
				ret = read(f, ch, 1);
			}
			return(0);
		}
	}
	return(0);
}

/*
-----------------------------------------------------------------------------
Function name:	skriv()
-----------------------------------------------------------------------------
Parameters:	char *s		Pointer to a string.'
Return value:	none
Called by:	none
Calls:		none
Description:	This function prints the decimal value of each character
		in a string. Used for debugging only.
-----------------------------------------------------------------------------
*/

void skriv(s)
char *s;
{
	int t=0;

	while(s[t] != '\0')
	{
		fprintf(stdout, " %3d", s[t++]);
	}
}

/*
-----------------------------------------------------------------------------
Function name: do_user()
-----------------------------------------------------------------------------
Parameters:	char *navn	Pointer to a string containing a username.
		char *bf	Pointer to a buffer. The buffer contains
				the articles subject.
		char *s		Pointer to a string which is the id and 
				subject of the article.
Return value:	none
Called by:	finn_stikkord()
Calls:		lock_file(), unlock_file, les_linje(), contains()
Description:	This function runs trough all of the lines in the file
		<user>.subjects to see if any of them matches the one
		in the bf buffer. If they match, the s string is written
		to the file <user>.articles together with the line and
		its priority.
-----------------------------------------------------------------------------
*/

void do_user(navn, bf, s)
char *navn;
char *bf;
char *s;
{
	int stikkord, innlegg, nfp;
	char ord[BUFSIZ+1];
	char ordfil[BUFSIZ+1], innleggfil[BUFSIZ+1];
	char setn[BUFSIZ+1];
	char newfile[BUFSIZ+1];
	char *tmp;

	sprintf(ordfil, "%s/%s.subjects", navn, navn);
	if((stikkord = open(ordfil, O_RDONLY|O_CREAT,  S_IREAD|S_IWRITE)) == -1) {
                fprintf(stderr, "%s: Couldn't open %s", myname, ordfil);
        	return;		
	}
	lock_file(stikkord);
	while(les_linje(stikkord, ord)) {
		tmp = ord + 1;
		if(ord[0] != '0') {
			if(contains(bf, tmp)) {
				sprintf(innleggfil, "%s/%s.articles", navn, navn);
				if((innlegg = open(innleggfil, O_WRONLY|O_APPEND|O_CREAT, S_IREAD|S_IWRITE)) == -1) {
              		       	        fprintf(stderr, "%s: Couldn't open %s", myname, innleggfil);
                       	 		return;	
                		}
				lock_file(innlegg);
				sprintf(setn, "[%s] [%c] %s\n",tmp,ord[0], s);

				write(innlegg, setn, strlen(setn));
				unlock_file(innlegg);
				close(innlegg);

				sprintf(newfile, "%s/%s.newnews", navn, navn);
				if((nfp = open(newfile, O_WRONLY|O_TRUNC|O_CREAT, S_IREAD|S_IWRITE)) == -1) {
                               		fprintf(stderr, "%s: Couldn't open %s",myname, innleggfil);
                                	return;	
                               	}
				lock_file(nfp);
				write(nfp, "YES\n", 4);
				unlock_file(nfp);
				close(nfp);
				unlock_file(stikkord);
				close(stikkord);
				return;
			}
		}	
	}
	unlock_file(stikkord);
	close(stikkord);
}

/*
-----------------------------------------------------------------------------
Function name: finn_stikkord() 
-----------------------------------------------------------------------------
Parameters:	char *bf		Contains the subject of an article
		char *s			Contains the id and subject of an
					article.
Return value:	none
Called by:	print_subject()
Calls:		lock_file(), unlock_file(), les_linje(), first_word(),
		do_user()
Description:	This function runs trough all the users and checks for each
		of them if they want to read the article.
-----------------------------------------------------------------------------
*/

void finn_stikkord(bf, s)
char *bf;
char *s;
{
	int brukere;
	char navn[BUFSIZ+1];
	
	if((brukere = open("agent.users", O_RDONLY|O_CREAT,  S_IREAD|S_IWRITE)) == -1) {
		fprintf(stderr, "%s: Couldn't open agent.users", myname);
		return;	
	}
	lock_file(brukere);
	while(les_linje(brukere, navn)) {
		first_word(navn);
		do_user(navn, bf, s);
	}
	unlock_file(brukere);
	close(brukere);

}

/*
-----------------------------------------------------------------------------
Function name: print_subject()
-----------------------------------------------------------------------------
Parameters:	char *s		Contains the id and subject of an article.
Return value:	none
Called by:	do_innlegg()
Calls:		finn_stikkord()
Description:	Get the subject from the string s and sends both the subject
		and s to finn_stikkord().
-----------------------------------------------------------------------------
*/

void print_subject(s)
char *s;
{
	int teller = 0;
	int t2 = 0;
	char bf[BUFSIZ+1];

	while(s[teller++] != 32);

	teller--;

	while(s[teller] != '\0') 
		bf[t2++] = s[teller++];

	bf[t2] = '\0';

	finn_stikkord(bf, s);
}

/*
-----------------------------------------------------------------------------
Function name: do_innlegg()
-----------------------------------------------------------------------------
Parameters:	int s		A valid socket.
Return value:	none
Called by:	main() (agentc)
Calls:		lock_file(), unlock_file(), get_article(), tcp_send()
		get_msg(), print_subject()
Description:	Reads every article id in New.News and gets the subject
		for that article from the NNTP-server(using the XHDR-
		command). The format recieved is: <id> subject. It passes
		the sring recieved to the print_subject().
-----------------------------------------------------------------------------
*/

void do_innlegg(s)
int s;
{
	int f;
	int l;
	int test;
	char command[80];
	char article[80];
	char *host;
	char *afile;
	char buf[BUFSIZ+1];

	test  = 1;

	if((f = open( "New.News", O_RDONLY)) == -1) {
		fprintf(stderr, "%s: Couldn't open New.News", myname);
                return;
	}

	lock_file(f);
	test = get_article(f, article);

	while(test) {
		sprintf( command, "xhdr subject %s\n", article);
		tcp_send(s, command);
		if(get_msg(s, '2', LINEFEED, buf, 3) == 1) {
			if(get_msg(s, '<', LINEFEED, buf, 3) == 1)	{
				print_subject(buf);
				get_msg(s, '2', LINEFEED, buf, 3);
			}
		}
		test = get_article(f, article);
	}
	unlock_file(f);
	close(f);
}
